@@ -2,16 +2,15 @@ from platform import platform
 import docker
 import requests
 import os
+import re
 import json
 import argparse
-import yaml
 import sys
 from time import sleep
+import logging
 
 project_name = "knuth33"
 docker_artifact_repository_loc = "us-central1-docker.pkg.dev"
-cluster_loc = "us-central1-a"
-cluster_name = "kluster-fuck"
 
 def printError(text):
     print(f"\033[0;31mERROR: {text}\033[00m")
@@ -27,9 +26,7 @@ def buildDocker(app_folder_path):
     docker_string = f"""FROM python:{py_version}
 COPY . .
 
-RUN pip3 install -r requirements.txt
-&& pip3 install python-multipart
-&& pip3 install uvicorn
+RUN pip3 install -r requirements.txt && pip3 install python-multipart && pip3 install uvicorn
 
 EXPOSE 8080
 CMD ["uvicorn", "main:app", "--host", "0.0.0.0", "--port", "8080"]
@@ -37,46 +34,13 @@ CMD ["uvicorn", "main:app", "--host", "0.0.0.0", "--port", "8080"]
     with open(app_folder_path+ "/Dockerfile", 'w') as f:
         f.write(docker_string)
 
-def buildYAML(imageArtifactRegistryLocation, model_name, image_namespace, version_name, app_folder_path, numReplicas, memUsageInGi):
-    numReplicas = numReplicas
-    yaml_data = {}
-
-    apiVersion = {"apiVersion": "apps/v1"} 
-    yaml_data.update(apiVersion)
-
-    # TODO: have this not just be auto set to 8Gi
-    memLimit = f'{memUsageInGi}Gi'
-    resourcesLimit = "256Mi"
-
-    deployment_name = model_name+"-"+version_name
-
-    kind = {"kind": "Deployment"}
-    yaml_data.update(kind)
-    metadata = {"metadata": {"name": (deployment_name),"namespace": image_namespace}, "labels":{"app":deployment_name,"version":version_name}}
-    yaml_data.update(metadata)
-    spec = {"spec": {"replicas": int(numReplicas),"selector": {"matchLabels":{"app": deployment_name}}, \
-            "template":{"metadata": {"labels": {"app": deployment_name}}, "spec": \
-            {"containers": [{"image" : f"{imageArtifactRegistryLocation}/{model_name}:{version_name}", "name" :deployment_name, "resources" : {"limits" : {"memory" : memLimit}}}]}}}}
-    yaml_data.update(spec)
-
-    returnString = yaml.dump(yaml_data, default_flow_style=False)
-
-    with open(app_folder_path+ "/app.yaml", 'w') as f:
-        f.write(returnString)
-
-def check_string(s):
-    if len(s) > 253:
-        return False
-    for c in s:
-        if not c.isalnum() and c not in ['-', '.']:
-            return False
-        elif c.isupper():
-            return False
-    return s[0].isalpha() and s[-1].isalnum()
+def stringTester(testString):
+    matched = re.fullmatch('[a-z0-9]([-a-z0-9]*[a-z0-9])', testString)
+    return(bool(matched))
 
 # NOTE: requirements is an optional parameter - if you specify it, we won't make you one. Also note that if we make one it'll
 # go next to your dockerfile, so make sure that the dockerfile knows where to find its requirements regardless 
-def deployModel(model_name, version_name, project_directory, number_pods, requirementsFile, dockerFile, memoryLimit, verbose_logging):
+def deployModel(model_name, version_name, project_directory, number_pods, requirementsFile, dockerFile, memoryLimit, verbose_logging, keep_alive):
     # TODO: consider doing this AFTER we successfully upload the model
     # TODO: look up model type 
     # TODO: return namespace name when we write it successfully
@@ -88,6 +52,7 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
         account_name = configObj["account_name"]
         account_id = configObj["account_id"]
     except Exception as e: 
+        logging.error(e, exc_info=True)
         printError("ERROR. Unable to verify credentials; re-run pyqai-init to reset user credentials.")
         return 
 
@@ -101,6 +66,9 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
     if(check_string(version_name)==False):
         printError("Error. The version name you provided, "+version_name+", does not match our naming convention. Please check our documentation for instructions: https://docs.pyqai.com/pyq/fundamentals/naming-conventions")
         return(False)
+    if(memoryLimit>16.0):
+        printError("Error. We currently do not support models larger than 16 GB in our alpha test. Please try again with a smaller model")
+        return(False)
 #endregion
 
 #region Authentication
@@ -125,6 +93,7 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
             printError(f"ERROR. Deployment failed. Authentication failed with error code: {getToken.status_code} and message: {getTokenResponse}")
             return
     except Exception as e:
+        logging.error(e, exc_info=True)
         printError(f"ERROR. Deployment failed. Authentication failed with error: {getToken.text} {e}")
         return
 
@@ -149,6 +118,7 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
             printError(f"ERROR. Deployment failed. Attempt to create user namespace failed with error code: {writeModels.status_code} and message: {writeModelsResponse}")
             return
     except Exception as e:
+        logging.error(e, exc_info=True)
         printError(f"ERROR. Deployment failed. Attempt to create user namespace failed with error: {writeModels.text} {e}")
         return
 #endregion
@@ -173,6 +143,7 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
             buildDocker(project_directory)
             printInfo("Dockerfile created successfully.")
         except Exception as e:
+            logging.error(e, exc_info=True)
             printError(f"ERROR. Deployment failed. Failed to create Dockerfile with error: {e}")
             return(False)
 
@@ -184,6 +155,7 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
         try: 
             os.system(f'pipreqs --force {project_directory}')
         except Exception as e:
+            logging.error(e, exc_info=True)
             printError(f"ERROR. Deployment failed. Failed to create requirements.txt with error: {e}")
             return(False)
 #endregion
@@ -202,6 +174,7 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
             printError(f"ERROR. Deployment failed. Attempt to create artifact repository failed with error code: {callCreateRepo.status_code} and message: {repo_location}")
             return
     except Exception as e:
+            logging.error(e, exc_info=True)
             printError(f"ERROR. Deployment failed. Attempt to create artifact repository failed with error: {e}")
             return
 #endregion
@@ -215,6 +188,7 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
         docker_client = docker.DockerClient()
         docker_client.login(username="_json_key",password=json.dumps(gcp_token),registry=f"https://us-central1-docker.pkg.dev")
     except Exception as e:
+        logging.error(e, exc_info=True)
         printError(f"ERROR. Unable to configure Docker. You may need to have the docker client running. Please install Docker Desktop, open the app on your computer and try again. If that doesn't work, contact us at team@pyqai.com. \nDeployment Failed with error: {e}")
         return
 
@@ -226,6 +200,7 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
         buildOutput = docker_client.images.build(path = project_directory, rm = True, tag = push_tag, platform="linux/amd64")
         if verbose_logging : printInfo(f"Docker build output: {buildOutput}")
     except Exception as e:
+        logging.error(e, exc_info=True)
         printError(f"ERROR. Deployment failed, failed build docker image with exception {e}.")
         return
 
@@ -241,9 +216,10 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
     gibibytes_memory_size = membibytes_memory_size / 1024
 
     if verbose_logging : printInfo(f"Docker image memory size in gibibytes: {gibibytes_memory_size}")
-
+    auto = 0
     mem_util = memoryLimit
     calculated_mem_util = 0
+    size = 0
 
     try:
         # run the container so that we can get some stats on it.
@@ -271,25 +247,20 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
             calculated_mem_util = gibibytes_mem_usage + gibibytes_memory_size
 
             printInfo(f"Memory requirement determined to be {calculated_mem_util}")
+
+        if(calculated_mem_util==0):
+            size = memoryLimit
+        else:
+            size = calculated_mem_util
+            auto = 1
+        if(size>16.0):
+            printError("Error. We currently do not support models larger than 16 GB in our alpha test. Please try again with a smaller model")
+            return(False)
     except Exception as e:
+        logging.error(e, exc_info=True)
         printError(f"ERROR. We were unable to determine the memory needs of your model, getting an error {e} \nPlease try and run your container locally to make sure it works, or provide a memory limit and redeploy.")
         return
 
-    if memoryLimit == None or memoryLimit <= 0: 
-        if(calculated_mem_util < 3.0):
-            mem_util = 4
-        elif(calculated_mem_util < 6.8):
-            mem_util = 8
-        elif(calculated_mem_util > 16.0):
-            printError("ERROR. Sorry, we only allow containers up to 16 GB in size for our alpha test. Please try again with a smaller model")
-            return(False)
-        else:
-            mem_util = 16
-    elif memoryLimit > 16:
-        printError("ERROR. Sorry, we only allow containers up to 16 GB in size for our alpha test. Please try again with a smaller model")
-        return(False)
-    else:
-        mem_util = memoryLimit
 #endregion
 
 #region Push docker container
@@ -309,6 +280,7 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
                 printError(f"ERROR. Something went wrong when trying to push the docker image, error detail: {line['errorDetail']}")
                 return
     except Exception as e:
+        logging.error(e, exc_info=True)
         printError(f"ERROR.Deployment failed, unable push docker image with exception {e}")
         return
     
@@ -317,33 +289,17 @@ def deployModel(model_name, version_name, project_directory, number_pods, requir
 
 #region create and expose workload
     printInfo("Creating and exposing kubernetes workload...")
-
-    # build the yaml
-    namespace_name = account_name+'-'+model_name
-    buildYAML(f"{docker_artifact_repository_loc}/{project_name}/{repo_location}", 
-        model_name, namespace_name, version_name, 
-        project_directory, number_pods, mem_util)
-
-    yaml_location = project_directory + "/app.yaml"
-
-    json_yaml = ""
-    with open(yaml_location) as f:
-        json_yaml = json.dumps(yaml.safe_load(f))
-
     # here we call into the google function that will create the deployment and expose the service.
-    create_service_json = {"project_name":project_name,
-    "cluster_loc":cluster_loc,
-    "cluster_name":cluster_name, \
-    "imageArtifactRegistryLocation":docker_artifact_repository_loc,
-    "model_name":model_name, \
+    create_service_json = {"model_name":model_name, \
     "model_id":model_id,
-    "image_namespace":namespace_name,
     "version_name":version_name,
     "num_replicas":number_pods, \
-    "mem_limit":"8Gi",
     "account_name":account_name,
     "account_id": account_id,
-    "yaml_json": json_yaml}
+    "size": size,
+    "keep_alive": keep_alive,
+    "repo": repo_location,
+    "auto": auto}
 
     if verbose_logging : printInfo("Create Service JSON  " + json.dumps(create_service_json))
 
@@ -366,10 +322,11 @@ def main():
     parser.add_argument("--version_name", help="The name you want to give this version of your model", required=True)
     parser.add_argument("--project_directory", help="The directory of your project. Put only what you need to run predictions using your model in it.", required=True)
     parser.add_argument("--number_pods", help="The number of pods of your deployment if you are providing your own", required=False, default=2)
+    parser.add_argument("--keep_alive", help="How many hours the model should remain active without any usage. Anything greater than 48 hours may incur additional cost. You will experience a cold start once this time period expires. Defaults to 48", required=False, default=48, type=int)
     parser.add_argument("--requirements", help="Did you create your own requirements.txt in your project directory or should we make you one? Answer True or False. Default False.", required=False, default="False", choices=["True","False"])
     parser.add_argument("--memory_limit", help="The memory limit for your deployment in Gi if you are providing your own, else we will attempt to guess. Provide only the number with decimals.", required=False, default=-999.9, type=float)
     parser.add_argument("--dockerfile", help="Did you create your own Dockerfile in your project directory or should we make you one? Answer True or False. Default False.", required=False, default="False", choices=["True","False"])
     parser.add_argument("--verbose_logging", help="Do you want a bunch of logs? Answer True or False. Default False.", required=False, default="False", choices=["True","False"])
     
     args = parser.parse_args()
-    deployModel(args.model_name, args.version_name, args.project_directory, args.number_pods, args.requirements, args.dockerfile, args.memory_limit, args.verbose_logging)
\ No newline at end of file
+    deployModel(args.model_name, args.version_name, args.project_directory, args.number_pods, args.requirements, args.dockerfile, args.memory_limit, args.verbose_logging, args.keep_alive)
\ No newline at end of file
